# -*- coding: utf-8 -*-

from tools import trace, timer

from View.ASubWindow import ASubMainWindow
from View.ListedSubWindow import ListedSubWindow

from PyQt5.QtGui import (
    QKeySequence,
)

from PyQt5.QtCore import (
    Qt, QVariant, QAbstractTableModel,
    QCoreApplication, QModelIndex, pyqtSlot,
    QRect,
)

from PyQt5.QtWidgets import (
    QDialogButtonBox, QPushButton, QLineEdit,
    QFileDialog, QTableView, QAbstractItemView,
    QUndoStack, QShortcut, QAction, QItemDelegate,
    QComboBox, QVBoxLayout, QHeaderView, QTabWidget,
)

from View.InitialConditions.UndoCommand import (
    SetCommand, AddCommand, DelCommand,
    SortCommand, MoveCommand, PasteCommand,
    DuplicateCommand,
)

from View.InitialConditions.Table import TableModel

from View.Plot.MplCanvas import MplCanvas
from View.InitialConditions.translate import *

_translate = QCoreApplication.translate


class InitialConditionsWindow(ASubMainWindow, ListedSubWindow):
    def __init__(self, title="Initial condition",
                 study=None, parent=None):
        title = title + " - " + study.name

        super(InitialConditionsWindow, self).__init__(
            name=title, ui="InitialConditions", parent=parent
        )

        self._study = study
        self._reach = study.river.current_reach()
        self._ics = self._study.river.initial_conditions.get(self._reach)

        self.setup_sc()
        self.setup_table()
        self.setup_graph()
        self.setup_connections()

        self.ui.setWindowTitle(title)

    def setup_sc(self):
        self._undo_stack = QUndoStack()

        self.undo_sc = QShortcut(QKeySequence.Undo, self)
        self.redo_sc = QShortcut(QKeySequence.Redo, self)
        self.copy_sc = QShortcut(QKeySequence.Copy, self)
        self.paste_sc = QShortcut(QKeySequence.Paste, self)

    def setup_table(self):
        table = self.find(QTableView, f"tableView")
        self._table = TableModel(
            river = self._study.river,
            reach = self._reach,
            undo = self._undo_stack,
        )
        table.setModel(self._table)

        table.setSelectionBehavior(QAbstractItemView.SelectRows)
        table.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
        table.setAlternatingRowColors(True)

    def setup_graph(self):
        print("TODO")
        # self.canvas_1 = MplCanvas(width=5, height=4, dpi=100)
        # self.canvas_1.setObjectName("canvas_1")
        # self.plot_layout_1 = self.find(QVBoxLayout, "verticalLayout_1")
        # self.plot_layout_1.addWidget(self.canvas_1)

        # self.plot = PlotXY(
        #     canvas = self.canvas_1,
        #     data = None,
        #     toolbar = None,
        # )

        # self.canvas_2 = MplCanvas(width=5, height=4, dpi=100)
        # self.canvas_2.setObjectName("canvas_2")
        # self.plot_layout_2 = self.find(QVBoxLayout, "verticalLayout_2")
        # self.plot_layout_2.addWidget(self.canvas_2)

        # self.plot = PlotXY(
        #     canvas = self.canvas_2,
        #     data = None,
        #     toolbar = None,
        # )

    def setup_connections(self):
        self.find(QAction, "action_add").triggered.connect(self.add)
        self.find(QAction, "action_del").triggered.connect(self.delete)
        self.find(QAction, "action_sort").triggered.connect(self.sort)

        self.undo_sc.activated.connect(self.undo)
        self.redo_sc.activated.connect(self.redo)
        self.copy_sc.activated.connect(self.copy)
        self.paste_sc.activated.connect(self.paste)

    def index_selected_row(self):
        table = self.find(QTableView, f"tableView")
        return table.selectionModel()\
                    .selectedRows()[0]\
                    .row()

    def index_selected_rows(self):
        table = self.find(QTableView, f"tableView")
        return list(
            # Delete duplicate
            set(
                map(
                    lambda i: i.row(),
                    table.selectedIndexes()
                )
            )
        )

    def add(self):
        rows = self.index_selected_rows()
        if len(self._ics) == 0 or len(rows) == 0:
            self._table.add(0)
        else:
            self._table.add(rows[0])

    def delete(self):
        rows = self.index_selected_rows()
        if len(rows) == 0:
            return

        self._table.delete(rows)

    def sort(self):
        self._table.sort(False)

    def move_up(self):
        row = self.index_selected_row()
        self._table.move_up(row)

    def move_down(self):
        row = self.index_selected_row()
        self._table.move_down(row)

    def copy(self):
        print("TODO")

    def paste(self):
        print("TODO")

    def undo(self):
        self._table.undo()

    def redo(self):
        self._table.redo()
